@@ -1,5 +1,5 @@
 // Copyright (c) Meta Platforms, Inc. and affiliates.
-// SPDX-License-Identifier: GPL-3.0-or-later
+// SPDX-License-Identifier: LGPL-2.1-or-later
 
 /**
  * @file
@@ -39,8 +39,8 @@
  * @{
  */
 
-/** State of a @ref drgn_debug_info_module. */
-enum drgn_debug_info_module_state {
+/** State of a @ref drgn_module. */
+enum drgn_module_state {
 	/** Reported but not indexed. */
 	DRGN_DEBUG_INFO_MODULE_NEW,
 	/** Reported and will be indexed on success. */
@@ -49,36 +49,6 @@ enum drgn_debug_info_module_state {
 	DRGN_DEBUG_INFO_MODULE_INDEXED,
 } __attribute__((__packed__));
 
-enum drgn_debug_info_scn {
-	/* Sections whose data we should cache when loading the module. */
-	DRGN_SCN_DEBUG_INFO,
-	DRGN_SCN_DEBUG_TYPES,
-	DRGN_SCN_DEBUG_ABBREV,
-	DRGN_SCN_DEBUG_STR,
-	DRGN_SCN_DEBUG_STR_OFFSETS,
-	DRGN_SCN_DEBUG_LINE,
-	DRGN_SCN_DEBUG_LINE_STR,
-
-	DRGN_NUM_DEBUG_SCN_DATA_PRECACHE,
-
-	/* Sections whose data we should cache when it is first used. */
-	DRGN_SCN_DEBUG_ADDR = DRGN_NUM_DEBUG_SCN_DATA_PRECACHE,
-	DRGN_SCN_DEBUG_FRAME,
-	DRGN_SCN_EH_FRAME,
-	DRGN_SCN_ORC_UNWIND_IP,
-	DRGN_SCN_ORC_UNWIND,
-	DRGN_SCN_DEBUG_LOC,
-	DRGN_SCN_DEBUG_LOCLISTS,
-
-	DRGN_NUM_DEBUG_SCN_DATA,
-
-	/* Sections whose data doesn't need to be cached. */
-	DRGN_SCN_TEXT = DRGN_NUM_DEBUG_SCN_DATA,
-	DRGN_SCN_GOT,
-
-	DRGN_NUM_DEBUG_SCNS,
-};
-
 /**
  * A module reported to a @ref drgn_debug_info.
  *
@@ -86,9 +56,9 @@ enum drgn_debug_info_scn {
  * not loaded).
  *
  * Files are identified by canonical path and, if present, build ID. Each (path,
- * address range) is uniquely represented by a @ref drgn_debug_info_module.
+ * address range) is uniquely represented by a @ref drgn_module.
  */
-struct drgn_debug_info_module {
+struct drgn_module {
 	/** @c NULL if the module does not have a build ID. */
 	const void *build_id;
 	/** Zero if the module does not have a build ID. */
@@ -99,21 +69,30 @@ struct drgn_debug_info_module {
 	char *name;
 
 	Dwfl_Module *dwfl_module;
-	struct drgn_platform platform;
-	Elf_Scn *scns[DRGN_NUM_DEBUG_SCNS];
-	Elf_Scn *alt_debug_info;
-	Elf_Scn *alt_debug_str;
-	Elf_Data *scn_data[DRGN_NUM_DEBUG_SCN_DATA];
-	Elf_Data *alt_debug_info_data;
-	Elf_Data *alt_debug_str_data;
+	/** File that is loaded into the program. */
+	struct drgn_elf_file *loaded_file;
+	/** File containing debugging information. */
+	struct drgn_elf_file *debug_file;
+	/**
+	 * Difference between addresses in program and addresses in @ref
+	 * drgn_module::loaded_file.
+	 */
+	uint64_t loaded_file_bias;
+	/**
+	 * Difference between addresses in program and addresses in @ref
+	 * drgn_module::debug_file.
+	 */
+	uint64_t debug_file_bias;
 
 	/** DWARF debugging information. */
-	struct drgn_dwarf_module_info dwarf;
+	struct drgn_module_dwarf_info dwarf;
 	/** ORC unwinder information. */
-	struct drgn_orc_module_info orc;
+	struct drgn_module_orc_info orc;
 
-	/** Whether .debug_frame and .eh_frame have been parsed. */
-	bool parsed_frames;
+	/** Whether DWARF CFI from .debug_frame has been parsed. */
+	bool parsed_debug_frame;
+	/** Whether EH CFI from .eh_frame has been parsed. */
+	bool parsed_eh_frame;
 	/** Whether ORC unwinder data has been parsed. */
 	bool parsed_orc;
 
@@ -125,7 +104,7 @@ struct drgn_debug_info_module {
 	char *path;
 	Elf *elf;
 	int fd;
-	enum drgn_debug_info_module_state state;
+	enum drgn_module_state state;
 	/** Error while loading. */
 	struct drgn_error *err;
 	/**
@@ -136,43 +115,10 @@ struct drgn_debug_info_module {
 	 * loading, all files with the same build ID and address range are
 	 * linked in a list. Only one is indexed; the rest are destroyed.
 	 */
-	struct drgn_debug_info_module *next;
+	struct drgn_module *next;
 };
 
-struct drgn_error *
-drgn_debug_info_module_cache_section(struct drgn_debug_info_module *module,
-				     enum drgn_debug_info_scn scn);
-
-struct drgn_error *
-drgn_error_debug_info_scn(struct drgn_debug_info_module *module,
-			  enum drgn_debug_info_scn scn, const char *ptr,
-			  const char *message);
-
-struct drgn_debug_info_buffer {
-	struct binary_buffer bb;
-	struct drgn_debug_info_module *module;
-	enum drgn_debug_info_scn scn;
-};
-
-struct drgn_error *drgn_debug_info_buffer_error(struct binary_buffer *bb,
-						const char *pos,
-						const char *message);
-
-static inline void
-drgn_debug_info_buffer_init(struct drgn_debug_info_buffer *buffer,
-			    struct drgn_debug_info_module *module,
-			    enum drgn_debug_info_scn scn)
-{
-	binary_buffer_init(&buffer->bb, module->scn_data[scn]->d_buf,
-			   module->scn_data[scn]->d_size,
-			   drgn_platform_is_little_endian(&module->platform),
-			   drgn_debug_info_buffer_error);
-	buffer->module = module;
-	buffer->scn = scn;
-}
-
-DEFINE_HASH_TABLE_TYPE(drgn_debug_info_module_table,
-		       struct drgn_debug_info_module *)
+DEFINE_HASH_TABLE_TYPE(drgn_module_table, struct drgn_module *)
 
 DEFINE_HASH_SET_TYPE(c_string_set, const char *)
 
@@ -184,12 +130,12 @@ struct drgn_debug_info {
 	/** DWARF frontend library handle. */
 	Dwfl *dwfl;
 	/** Modules keyed by build ID and address range. */
-	struct drgn_debug_info_module_table modules;
+	struct drgn_module_table modules;
 	/**
 	 * Names of indexed modules.
 	 *
-	 * The entries in this set are @ref drgn_debug_info_module::name, so
-	 * they should not be freed.
+	 * The entries in this set are @ref drgn_module::name, so they should
+	 * not be freed.
 	 */
 	struct c_string_set module_names;
 	/** DWARF debugging information. */
@@ -203,8 +149,7 @@ struct drgn_error *drgn_debug_info_create(struct drgn_program *prog,
 /** Destroy a @ref drgn_debug_info. */
 void drgn_debug_info_destroy(struct drgn_debug_info *dbinfo);
 
-DEFINE_VECTOR_TYPE(drgn_debug_info_module_vector,
-		   struct drgn_debug_info_module *)
+DEFINE_VECTOR_TYPE(drgn_module_vector, struct drgn_module *)
 
 /** State tracked while loading debugging information. */
 struct drgn_debug_info_load_state {
@@ -214,7 +159,7 @@ struct drgn_debug_info_load_state {
 	const bool load_default;
 	const bool load_main;
 	/** Newly added modules to be indexed. */
-	struct drgn_debug_info_module_vector new_modules;
+	struct drgn_module_vector new_modules;
 	/** Formatted errors reported by @ref drgn_debug_info_report_error(). */
 	struct string_builder errors;
 	/** Number of errors reported by @ref drgn_debug_info_report_error(). */
@@ -309,11 +254,12 @@ drgn_debug_info_find_object(const char *name, size_t name_len,
 			    struct drgn_object *ret);
 
 /**
- * Get the Call Frame Information in a @ref drgn_debug_info_module at a given
- * program counter.
+ * Get the Call Frame Information in a @ref drgn_module at a given program
+ * counter.
  *
  * @param[in] module Module containing @p pc.
  * @param[in] pc Program counter.
+ * @param[out] file_ret Returned file containing CFI.
  * @param[in,out] row_ret Returned CFI row.
  * @param[out] interrupted_ret Whether the found frame interrupted its caller.
  * @param[out] ret_addr_regno_ret Returned return address register number.
@@ -321,23 +267,16 @@ drgn_debug_info_find_object(const char *name, size_t name_len,
  * drgn_not_found if CFI wasn't found.
  */
 struct drgn_error *
-drgn_debug_info_module_find_cfi(struct drgn_program *prog,
-				struct drgn_debug_info_module *module,
-				uint64_t pc, struct drgn_cfi_row **row_ret,
-				bool *interrupted_ret,
-				drgn_register_number *ret_addr_regno_ret);
+drgn_module_find_cfi(struct drgn_program *prog, struct drgn_module *module,
+		     uint64_t pc, struct drgn_elf_file **file_ret,
+		     struct drgn_cfi_row **row_ret, bool *interrupted_ret,
+		     drgn_register_number *ret_addr_regno_ret);
 
 struct drgn_error *open_elf_file(const char *path, int *fd_ret, Elf **elf_ret);
 
 struct drgn_error *find_elf_file(char **path_ret, int *fd_ret, Elf **elf_ret,
 				 const char * const *path_formats, ...);
 
-/*
- * NB: if the section is SHT_NOBITS, this returns an Elf_Data with d_buf = NULL
- * and d_size >= 0.
- */
-struct drgn_error *read_elf_section(Elf_Scn *scn, Elf_Data **ret);
-
 struct drgn_error *elf_address_range(Elf *elf, uint64_t bias,
 				     uint64_t *start_ret, uint64_t *end_ret);
 
