@@ -1,5 +1,5 @@
 // Copyright (c) Meta Platforms, Inc. and affiliates.
-// SPDX-License-Identifier: GPL-3.0-or-later
+// SPDX-License-Identifier: LGPL-2.1-or-later
 
 /**
  * @file
@@ -25,10 +25,11 @@
 /**
  * @mainpage
  *
- * libdrgn implements the core of <a
- * href="https://github.com/osandov/drgn">drgn</a>, a debugger-as-a-library. It
- * implements the main drgn abstractions: @ref Programs, @ref Types, and @ref
- * Objects. See <a href="modules.html">Modules</a> for detailed documentation.
+ * libdrgn provides the functionality of the <a
+ * href="https://github.com/osandov/drgn">drgn</a> programmable debugger as a
+ * library. It implements the main drgn abstractions: @ref Programs, @ref Types,
+ * and @ref Objects. See <a href="modules.html">Modules</a> for detailed
+ * documentation.
  */
 
 /** Major version of drgn. */
@@ -36,7 +37,7 @@
 /** Minor version of drgn. */
 #define DRGN_VERSION_MINOR 0
 /** Patch level of drgn. */
-#define DRGN_VERSION_PATCH 19
+#define DRGN_VERSION_PATCH 22
 
 /**
  * @defgroup ErrorHandling Error handling
@@ -230,18 +231,23 @@ struct drgn_error *drgn_error_copy(struct drgn_error *src)
 	__attribute__((__returns_nonnull__));
 
 /**
- * Write a @ref drgn_error to a @c stdio stream.
- *
- * For @ref DRGN_ERROR_OS errors, this concatenates @ref drgn_error::message,
- * @ref drgn_error::path, and @c strerror() of @ref drgn_error::errnum.
- * Otherwise, this just writes @c message.
+ * Write a @ref drgn_error followed by a newline to a @c stdio stream.
  *
  * @param[in] file File to write to (usually @c stderr).
  * @param[in] err Error to write.
- * @return Non-negative on success, @c EOF on failure.
+ * @return Non-negative on success, negative on failure.
  */
 int drgn_error_fwrite(FILE *file, struct drgn_error *err);
 
+/**
+ * Write a @ref drgn_error followed by a newline to a file descriptor.
+ *
+ * @param[in] fd File descriptor to write to.
+ * @param[in] err Error to write.
+ * @return Non-negative on success, negative on failure.
+ */
+int drgn_error_dwrite(int fd, struct drgn_error *err);
+
 /**
  * Free a @ref drgn_error.
  *
@@ -369,6 +375,8 @@ enum drgn_architecture {
 	DRGN_ARCH_PPC64,
 	DRGN_ARCH_RISCV64,
 	DRGN_ARCH_RISCV32,
+	DRGN_ARCH_S390X,
+	DRGN_ARCH_S390,
 };
 
 /** Flags describing a @ref drgn_platform. */
@@ -2708,6 +2716,9 @@ struct drgn_stack_trace;
 /** Destroy a @ref drgn_stack_trace. */
 void drgn_stack_trace_destroy(struct drgn_stack_trace *trace);
 
+/** Get the @ref drgn_program that a @ref drgn_stack_trace came from. */
+struct drgn_program *drgn_stack_trace_program(struct drgn_stack_trace *trace);
+
 /** Get the number of stack frames in a stack trace. */
 size_t drgn_stack_trace_num_frames(struct drgn_stack_trace *trace);
 
@@ -2769,6 +2780,15 @@ const char *drgn_stack_frame_source(struct drgn_stack_trace *trace,
 bool drgn_stack_frame_pc(struct drgn_stack_trace *trace, size_t frame,
 			 uint64_t *ret);
 
+/**
+ * Get the stack pointer at a stack frame.
+ *
+ * @param[out] ret Returned stack pointer.
+ * @return @c true if the stack pointer is known, @c false if it is not.
+ */
+bool drgn_stack_frame_sp(struct drgn_stack_trace *trace, size_t frame,
+			 uint64_t *ret);
+
 /**
  * Get the function symbol at a stack frame.
  *
@@ -2780,6 +2800,31 @@ struct drgn_error *drgn_stack_frame_symbol(struct drgn_stack_trace *trace,
 					   size_t frame,
 					   struct drgn_symbol **ret);
 
+/**
+ * Get the names of local objects in the scope of this frame.
+ *
+ * The array of names must be freed with @ref drgn_stack_frame_locals_destroy().
+ *
+ * @param[out] names_ret Returned array of names. On success, must be freed with
+ * @ref drgn_stack_frame_locals_destroy().
+ * @param[out] count_ret Returned number of names in @p names_ret.
+ * @return @c NULL on success, non-@c NULL on error
+ */
+struct drgn_error *
+drgn_stack_frame_locals(struct drgn_stack_trace *trace, size_t frame,
+			const char ***names_ret, size_t *count_ret);
+
+/**
+ * Free an array of names returned by @ref drgn_stack_frame_locals().
+ *
+ * The individual names from this array are invalid once this function is
+ * called. Any string which will be used later should be copied.
+ *
+ * @param names Array of names returned by @ref drgn_stack_frame_locals().
+ * @param count Count returned by @ref drgn_stack_frame_locals().
+ */
+void drgn_stack_frame_locals_destroy(const char **names, size_t count);
+
 /**
  * Find an object in the scope of a stack frame.
  *
