@@ -1,10 +1,11 @@
 // Copyright (c) Meta Platforms, Inc. and affiliates.
-// SPDX-License-Identifier: GPL-3.0-or-later
+// SPDX-License-Identifier: LGPL-2.1-or-later
 
 #include <errno.h>
 #include <getopt.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdnoreturn.h>
 #include <time.h>
 #include <unistd.h>
 
@@ -25,7 +26,7 @@ static inline struct timespec timespec_sub(struct timespec a, struct timespec b)
 	}
 }
 
-static void usage(bool error)
+noreturn static void usage(bool error)
 {
 	fprintf(error ? stderr : stdout,
 		"usage: load_debug_info [-k|-c CORE|-p PID] [PATH...]\n"
