@@ -1,5 +1,5 @@
 // Copyright (c) Meta Platforms, Inc. and affiliates.
-// SPDX-License-Identifier: GPL-3.0-or-later
+// SPDX-License-Identifier: LGPL-2.1-or-later
 
 #include <assert.h>
 #include <ctype.h>
@@ -536,14 +536,13 @@ static struct drgn_error *
 c_format_type_name(struct drgn_qualified_type qualified_type, char **ret)
 {
 	struct drgn_error *err;
-	struct string_builder sb = {};
-
+	struct string_builder sb = STRING_BUILDER_INIT;
 	err = c_format_type_name_impl(qualified_type, &sb);
 	if (err) {
 		free(sb.str);
 		return err;
 	}
-	if (!string_builder_finalize(&sb, ret))
+	if (!(*ret = string_builder_null_terminate(&sb)))
 		return &drgn_enomem;
 	return NULL;
 }
@@ -552,8 +551,7 @@ static struct drgn_error *
 c_format_type(struct drgn_qualified_type qualified_type, char **ret)
 {
 	struct drgn_error *err;
-	struct string_builder sb = {};
-
+	struct string_builder sb = STRING_BUILDER_INIT;
 	if (drgn_type_is_complete(qualified_type.type))
 		err = c_define_type(qualified_type, 0, &sb);
 	else
@@ -562,7 +560,7 @@ c_format_type(struct drgn_qualified_type qualified_type, char **ret)
 		free(sb.str);
 		return err;
 	}
-	if (!string_builder_finalize(&sb, ret))
+	if (!(*ret = string_builder_null_terminate(&sb)))
 		return &drgn_enomem;
 	return NULL;
 }
@@ -1604,15 +1602,14 @@ static struct drgn_error *c_format_object(const struct drgn_object *obj,
 					  char **ret)
 {
 	struct drgn_error *err;
-	struct string_builder sb = {};
-
+	struct string_builder sb = STRING_BUILDER_INIT;
 	err = c_format_object_impl(obj, 0, columns, max(columns, (size_t)1),
 				   flags, &sb);
 	if (err) {
 		free(sb.str);
 		return err;
 	}
-	if (!string_builder_finalize(&sb, ret))
+	if (!(*ret = string_builder_null_terminate(&sb)))
 		return &drgn_enomem;
 	return NULL;
 }
@@ -1653,6 +1650,7 @@ enum {
 	C_TOKEN_DOT,
 	C_TOKEN_NUMBER,
 	C_TOKEN_IDENTIFIER,
+	C_TOKEN_TEMPLATE_ARGUMENTS,
 };
 
 #include "c_keywords.inc"
@@ -1699,6 +1697,40 @@ struct drgn_error *drgn_c_family_lexer_func(struct drgn_lexer *lexer,
 		token->kind = C_TOKEN_DOT;
 		p++;
 		break;
+	case '<':
+		// This is a hack for
+		// cpp_append_template_arguments_to_identifier(). We don't want
+		// to deal with actually parsing template arguments, and we
+		// don't care about "<" otherwise, so this scans a token from
+		// the "<" to its matching ">".
+		if (cpp) {
+			token->kind = C_TOKEN_TEMPLATE_ARGUMENTS;
+			p++;
+			size_t less_thans = 1;
+			bool in_single_quotes = false;
+			do {
+				switch (*p++) {
+				case '<':
+					if (!in_single_quotes)
+						less_thans++;
+					break;
+				case '>':
+					if (!in_single_quotes)
+						less_thans--;
+					break;
+				case '\'':
+					// Handling the edge-case of an escaped single-quote
+					if (!(in_single_quotes && *(p - 2) == '\\'))
+						in_single_quotes = !in_single_quotes;
+					break;
+				case '\0':
+					return drgn_error_create(DRGN_ERROR_SYNTAX,
+								 "invalid template arguments");
+				}
+			} while (less_thans > 0);
+			break;
+		}
+		fallthrough;
 	default:
 		if (isalpha(*p) || *p == '_') {
 			do {
@@ -2063,6 +2095,37 @@ out:
 	return primitive;
 }
 
+// The DWARF index currently includes template arguments in indexed names. So,
+// to be able to find a type with template arguments, we have to look it up with
+// the template arguments included. This looks for a C_TOKEN_TEMPLATE_ARGUMENTS
+// token after the identifier and returns the length from the beginning of the
+// identifier to the end of the template arguments.
+//
+// Note that this requires that the user formats the template arguments exactly
+// as they appear in DWARF (which can vary between compilers). In the future, it
+// might be better to properly parse and either normalize the template arguments
+// or look them up as an AST somehow.
+static struct drgn_error *
+cpp_append_template_arguments_to_identifier(struct drgn_lexer *lexer,
+					    const char *identifier,
+					    size_t *len_ret)
+{
+	struct drgn_error *err;
+
+	// Only for C++.
+	if (!((struct drgn_c_family_lexer *)lexer)->cpp)
+		return NULL;
+
+	struct drgn_token token;
+	err = drgn_lexer_pop(lexer, &token);
+	if (err)
+		return err;
+	if (token.kind != C_TOKEN_TEMPLATE_ARGUMENTS)
+		return drgn_lexer_push(lexer, &token);
+	*len_ret = token.value + token.len - identifier;
+	return NULL;
+}
+
 static struct drgn_error *
 c_parse_specifier_qualifier_list(struct drgn_program *prog,
 				 struct drgn_lexer *lexer, const char *filename,
@@ -2114,6 +2177,11 @@ c_parse_specifier_qualifier_list(struct drgn_program *prog,
 			   specifier == SPECIFIER_NONE && !identifier) {
 			identifier = token.value;
 			identifier_len = token.len;
+			err = cpp_append_template_arguments_to_identifier(lexer,
+									  identifier,
+									  &identifier_len);
+			if (err)
+				return err;
 		} else if (token.kind == C_TOKEN_STRUCT ||
 			   token.kind == C_TOKEN_UNION ||
 			   token.kind == C_TOKEN_CLASS ||
@@ -2141,6 +2209,11 @@ c_parse_specifier_qualifier_list(struct drgn_program *prog,
 			}
 			identifier = token.value;
 			identifier_len = token.len;
+			err = cpp_append_template_arguments_to_identifier(lexer,
+									  identifier,
+									  &identifier_len);
+			if (err)
+				return err;
 		} else {
 			err = drgn_lexer_push(lexer, &token);
 			if (err)
@@ -2161,7 +2234,9 @@ c_parse_specifier_qualifier_list(struct drgn_program *prog,
 		} else if (tag_token == C_TOKEN_ENUM) {
 			kind = DRGN_TYPE_ENUM;
 		} else if (identifier) {
-			if (strstartswith(identifier, "size_t")) {
+			if (identifier_len == sizeof("size_t") - 1 &&
+			    memcmp(identifier, "size_t",
+				   sizeof("size_t") - 1) == 0) {
 				err = drgn_program_find_primitive_type(prog,
 								       DRGN_C_TYPE_SIZE_T,
 								       &ret->type);
@@ -2169,7 +2244,9 @@ c_parse_specifier_qualifier_list(struct drgn_program *prog,
 					return err;
 				ret->qualifiers = 0;
 				goto out;
-			} else if (strstartswith(identifier, "ptrdiff_t")) {
+			} else if (identifier_len == sizeof("ptrdiff_t") - 1 &&
+				   memcmp(identifier, "ptrdiff_t",
+					  sizeof("ptrdiff_t") - 1) == 0) {
 				err = drgn_program_find_primitive_type(prog,
 								       DRGN_C_TYPE_PTRDIFF_T,
 								       &ret->type);
