@@ -1,5 +1,5 @@
 // Copyright (c) Meta Platforms, Inc. and affiliates.
-// SPDX-License-Identifier: GPL-3.0-or-later
+// SPDX-License-Identifier: LGPL-2.1-or-later
 
 #include <assert.h>
 #include <byteswap.h>
@@ -21,6 +21,7 @@
 #include "debug_info.h"
 #include "error.h"
 #include "helpers.h"
+#include "io.h"
 #include "language.h"
 #include "linux_kernel.h"
 #include "memory_reader.h"
@@ -28,8 +29,8 @@
 #include "object_index.h"
 #include "program.h"
 #include "symbol.h"
-#include "vector.h"
 #include "util.h"
+#include "vector.h"
 
 static inline uint32_t drgn_thread_to_key(const struct drgn_thread *entry)
 {
@@ -205,23 +206,11 @@ static struct drgn_error *has_kdump_signature(const char *path, int fd,
 					      bool *ret)
 {
 	char signature[KDUMP_SIG_LEN];
-	size_t n = 0;
-
-	while (n < sizeof(signature)) {
-		ssize_t sret;
-
-		sret = pread(fd, signature + n, sizeof(signature) - n, n);
-		if (sret == -1) {
-			if (errno == EINTR)
-				continue;
-			return drgn_error_create_os("pread", errno, path);
-		} else if (sret == 0) {
-			*ret = false;
-			return NULL;
-		}
-		n += sret;
-	}
-	*ret = memcmp(signature, KDUMP_SIGNATURE, sizeof(signature)) == 0;
+	ssize_t r = pread_all(fd, signature, sizeof(signature), 0);
+	if (r < 0)
+		return drgn_error_create_os("pread", errno, path);
+	*ret = (r == sizeof(signature)
+		&& memcmp(signature, KDUMP_SIGNATURE, sizeof(signature)) == 0);
 	return NULL;
 }
 
@@ -433,14 +422,38 @@ drgn_program_set_core_dump(struct drgn_program *prog, const char *path)
 		prog->file_segments[j].file_size = phdr->p_filesz;
 		prog->file_segments[j].fd = prog->core_fd;
 		prog->file_segments[j].eio_is_fault = false;
+		/*
+		 * p_filesz < p_memsz is ambiguous for core dumps. The ELF
+		 * specification says that "if the segment's memory size p_memsz
+		 * is larger than the file size p_filesz, the 'extra' bytes are
+		 * defined to hold the value 0 and to follow the segment's
+		 * initialized area."
+		 *
+		 * However, the Linux kernel generates userspace core dumps with
+		 * segments with p_filesz < p_memsz to indicate that the range
+		 * between p_filesz and p_memsz was filtered out (see
+		 * coredump_filter in core(5)). These bytes were not necessarily
+		 * zeroes in the process's memory, which contradicts the ELF
+		 * specification in a way.
+		 *
+		 * As of Linux 5.19, /proc/kcore and /proc/vmcore never have
+		 * segments with p_filesz < p_memsz. However, makedumpfile
+		 * creates segments with p_filesz < p_memsz to indicate ranges
+		 * that were excluded. This is similar to Linux userspace core
+		 * dumps, except that makedumpfile can also exclude ranges that
+		 * were all zeroes.
+		 *
+		 * So, for userspace core dumps, we want to fault for ranges
+		 * between p_filesz and p_memsz to indicate that the memory was
+		 * not saved rather than lying and returning zeroes. For
+		 * /proc/kcore, we don't expect to see p_filesz < p_memsz but we
+		 * fault to be safe. For Linux kernel core dumps, we can't
+		 * distinguish between memory that was excluded because it was
+		 * all zeroes and memory that was excluded by makedumpfile for
+		 * another reason, so we're forced to always return zeroes.
+		 */
+		prog->file_segments[j].zerofill = vmcoreinfo_note && !is_proc_kcore;
 		err = drgn_program_add_memory_segment(prog, phdr->p_vaddr,
-						      /*
-						       * Don't override the page
-						       * table reader for
-						       * unsaved regions.
-						       */
-						      pgtable_reader ?
-						      phdr->p_filesz :
 						      phdr->p_memsz,
 						      drgn_read_memory_file,
 						      &prog->file_segments[j],
@@ -451,8 +464,6 @@ drgn_program_set_core_dump(struct drgn_program *prog, const char *path)
 		    phdr->p_paddr != (is_64_bit ? UINT64_MAX : UINT32_MAX)) {
 			err = drgn_program_add_memory_segment(prog,
 							      phdr->p_paddr,
-							      pgtable_reader ?
-							      phdr->p_filesz :
 							      phdr->p_memsz,
 							      drgn_read_memory_file,
 							      &prog->file_segments[j],
@@ -533,7 +544,7 @@ drgn_program_set_core_dump(struct drgn_program *prog, const char *path)
 		err = drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,
 					"unrecognized QEMU memory dump; "
 					"for Linux guests, run QEMU with '-device vmcoreinfo', "
-					"compile the kernel with CONFIG_CRASH_CORE and CONFIG_FW_CFG, "
+					"compile the kernel with CONFIG_FW_CFG_SYSFS and CONFIG_KEXEC, "
 					"and load the qemu_fw_cfg kernel module "
 					"before dumping the guest memory "
 					"(requires Linux >= 4.17 and QEMU >= 2.11)");
@@ -606,6 +617,7 @@ drgn_program_set_pid(struct drgn_program *prog, pid_t pid)
 	prog->file_segments[0].file_size = UINT64_MAX;
 	prog->file_segments[0].fd = prog->core_fd;
 	prog->file_segments[0].eio_is_fault = true;
+	prog->file_segments[0].zerofill = false;
 	err = drgn_program_add_memory_segment(prog, 0, UINT64_MAX,
 					      drgn_read_memory_file,
 					      prog->file_segments, false);
