@@ -1,5 +1,5 @@
 // Copyright (c) Meta Platforms, Inc. and affiliates.
-// SPDX-License-Identifier: GPL-3.0-or-later
+// SPDX-License-Identifier: LGPL-2.1-or-later
 
 #ifndef DRGNPY_H
 #define DRGNPY_H
@@ -39,6 +39,33 @@
 
 #define DRGNPY_PUBLIC __attribute__((__visibility__("default")))
 
+// PyLong_From* and PyLong_As* for stdint.h types. These use _Generic for
+// slightly more type safety (e.g., so you can't pass an int64_t to
+// PyLong_FromUint64()).
+#if ULONG_MAX == UINT64_MAX
+#define PyLong_FromUint64(v) _Generic((v), uint64_t: PyLong_FromUnsignedLong)(v)
+#define PyLong_AsUint64(obj) ((uint64_t)PyLong_AsUnsignedLong(obj))
+#define PyLong_AsUint64Mask(obj) ((uint64_t)PyLong_AsUnsignedLongMask(obj))
+#elif ULLONG_MAX == UINT64_MAX
+#define PyLong_FromUint64(v) _Generic((v), uint64_t: PyLong_FromUnsignedLongLong)(v)
+#define PyLong_AsUint64(obj) ((uint64_t)PyLong_AsUnsignedLongLong(obj))
+#define PyLong_AsUint64Mask(obj) ((uint64_t)PyLong_AsUnsignedLongLongMask(obj))
+#endif
+
+#if LONG_MIN == INT64_MIN && LONG_MAX == INT64_MAX
+#define PyLong_FromInt64(v) _Generic((v), int64_t: PyLong_FromLong)(v)
+#define PyLong_AsInt64(obj) ((int64_t)PyLong_AsLong(obj))
+#elif LLONG_MIN == INT64_MIN && LLONG_MAX == INT64_MAX
+#define PyLong_FromInt64(v) _Generic((v), int64_t: PyLong_FromLongLong)(v)
+#define PyLong_AsInt64(obj) ((int64_t)PyLong_AsLongLong(obj))
+#endif
+
+#if ULONG_MAX >= UINT32_MAX
+#define PyLong_FromUint32(v) _Generic((v), uint32_t: PyLong_FromUnsignedLong)(v)
+#define PyLong_FromUint16(v) _Generic((v), uint16_t: PyLong_FromUnsignedLong)(v)
+#define PyLong_FromUint8(v) _Generic((v), uint8_t: PyLong_FromUnsignedLong)(v)
+#endif
+
 #define Py_RETURN_BOOL(cond) do {	\
 	if (cond)			\
 		Py_RETURN_TRUE;		\
@@ -205,15 +232,15 @@ void *set_drgn_error(struct drgn_error *err);
 void *set_error_type_name(const char *format,
 			  struct drgn_qualified_type qualified_type);
 
+#define call_tp_alloc(type) ((type *)type##_type.tp_alloc(&type##_type, 0))
+
 PyObject *Language_wrap(const struct drgn_language *language);
 int language_converter(PyObject *o, void *p);
 int add_languages(void);
 
 static inline DrgnObject *DrgnObject_alloc(Program *prog)
 {
-	DrgnObject *ret;
-
-	ret = (DrgnObject *)DrgnObject_type.tp_alloc(&DrgnObject_type, 0);
+	DrgnObject *ret = call_tp_alloc(DrgnObject);
 	if (ret) {
 		drgn_object_init(&ret->obj, &prog->prog);
 		Py_INCREF(prog);
@@ -267,6 +294,8 @@ DrgnType *Program_function_type(Program *self, PyObject *args, PyObject *kwds);
 int append_string(PyObject *parts, const char *s);
 int append_format(PyObject *parts, const char *format, ...);
 PyObject *join_strings(PyObject *parts);
+// Implementation of _repr_pretty_() for IPython/Jupyter that just calls str().
+PyObject *repr_pretty_from_str(PyObject *self, PyObject *args, PyObject *kwds);
 
 struct index_arg {
 	bool allow_none;
@@ -279,13 +308,12 @@ struct index_arg {
 };
 int index_converter(PyObject *o, void *p);
 
-/* Helpers for path arguments based on posixmodule.c in CPython. */
 struct path_arg {
 	bool allow_none;
 	char *path;
 	Py_ssize_t length;
 	PyObject *object;
-	PyObject *cleanup;
+	PyObject *bytes;
 };
 int path_converter(PyObject *o, void *p);
 void path_cleanup(struct path_arg *path);
@@ -305,9 +333,10 @@ DrgnObject *drgnpy_linux_helper_per_cpu_ptr(PyObject *self, PyObject *args,
 					    PyObject *kwds);
 DrgnObject *drgnpy_linux_helper_idle_task(PyObject *self, PyObject *args,
 					  PyObject *kwds);
-DrgnObject *drgnpy_linux_helper_radix_tree_lookup(PyObject *self,
-						  PyObject *args,
-						  PyObject *kwds);
+PyObject *drgnpy_linux_helper_task_cpu(PyObject *self, PyObject *args,
+				       PyObject *kwds);
+DrgnObject *drgnpy_linux_helper_xa_load(PyObject *self, PyObject *args,
+					PyObject *kwds);
 DrgnObject *drgnpy_linux_helper_idr_find(PyObject *self, PyObject *args,
 					 PyObject *kwds);
 DrgnObject *drgnpy_linux_helper_find_pid(PyObject *self, PyObject *args,
