@@ -1,5 +1,5 @@
 // Copyright (c) Meta Platforms, Inc. and affiliates.
-// SPDX-License-Identifier: GPL-3.0-or-later
+// SPDX-License-Identifier: LGPL-2.1-or-later
 
 #include "drgnpy.h"
 #include "../helpers.h"
@@ -16,7 +16,7 @@ PyObject *drgnpy_linux_helper_direct_mapping_offset(PyObject *self, PyObject *ar
 	err = linux_helper_direct_mapping_offset(&((Program *)arg)->prog, &ret);
 	if (err)
 		return set_drgn_error(err);
-	return PyLong_FromUnsignedLongLong(ret);
+	return PyLong_FromUint64(ret);
 }
 
 PyObject *drgnpy_linux_helper_read_vm(PyObject *self, PyObject *args,
@@ -100,26 +100,39 @@ DrgnObject *drgnpy_linux_helper_idle_task(PyObject *self, PyObject *args,
 	return res;
 }
 
-DrgnObject *drgnpy_linux_helper_radix_tree_lookup(PyObject *self,
-						  PyObject *args,
-						  PyObject *kwds)
+PyObject *drgnpy_linux_helper_task_cpu(PyObject *self, PyObject *args,
+				       PyObject *kwds)
 {
-	static char *keywords[] = {"root", "index", NULL};
+	static char *keywords[] = {"task", NULL};
 	struct drgn_error *err;
-	DrgnObject *root;
-	struct index_arg index = {};
-	DrgnObject *res;
+	DrgnObject *task;
 
-	if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!O&:radix_tree_lookup",
-					 keywords, &DrgnObject_type, &root,
-					 index_converter, &index))
+	if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!:task_cpu", keywords,
+					 &DrgnObject_type, &task))
+		return NULL;
+	uint64_t cpu;
+	err = linux_helper_task_cpu(&task->obj, &cpu);
+	if (err)
+		return set_drgn_error(err);
+	return PyLong_FromUint64(cpu);
+}
+
+DrgnObject *drgnpy_linux_helper_xa_load(PyObject *self, PyObject *args,
+					PyObject *kwds)
+{
+	static char *keywords[] = {"xa", "index", NULL};
+	struct drgn_error *err;
+	DrgnObject *xa;
+	struct index_arg index = {};
+	if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!O&:xa_load", keywords,
+					 &DrgnObject_type, &xa, index_converter,
+					 &index))
 		return NULL;
 
-	res = DrgnObject_alloc(DrgnObject_prog(root));
+	DrgnObject *res = DrgnObject_alloc(DrgnObject_prog(xa));
 	if (!res)
 		return NULL;
-	err = linux_helper_radix_tree_lookup(&res->obj, &root->obj,
-					     index.uvalue);
+	err = linux_helper_xa_load(&res->obj, &xa->obj, index.uvalue);
 	if (err) {
 		Py_DECREF(res);
 		return set_drgn_error(err);
@@ -294,7 +307,7 @@ PyObject *drgnpy_linux_helper_kaslr_offset(PyObject *self, PyObject *args,
 
 	if (!(prog->prog.flags & DRGN_PROGRAM_IS_LINUX_KERNEL))
 		return PyErr_Format(PyExc_ValueError, "not Linux kernel");
-	return PyLong_FromUnsignedLongLong(prog->prog.vmcoreinfo.kaslr_offset);
+	return PyLong_FromUint64(prog->prog.vmcoreinfo.kaslr_offset);
 }
 
 PyObject *drgnpy_linux_helper_pgtable_l5_enabled(PyObject *self, PyObject *args,
