@@ -1,5 +1,5 @@
 // Copyright (c) Meta Platforms, Inc. and affiliates.
-// SPDX-License-Identifier: GPL-3.0-or-later
+// SPDX-License-Identifier: LGPL-2.1-or-later
 
 #include <stdarg.h>
 
@@ -23,8 +23,7 @@ static const char *drgn_type_kind_str(struct drgn_type *type)
 
 DRGNPY_PUBLIC PyObject *DrgnType_wrap(struct drgn_qualified_type qualified_type)
 {
-	DrgnType *type_obj = (DrgnType *)DrgnType_type.tp_alloc(&DrgnType_type,
-								0);
+	DrgnType *type_obj = call_tp_alloc(DrgnType);
 	if (!type_obj)
 		return NULL;
 	type_obj->type = qualified_type.type;
@@ -118,7 +117,7 @@ static PyObject *DrgnType_get_size(DrgnType *self)
 	}
 	if (!drgn_type_is_complete(self->type))
 		Py_RETURN_NONE;
-	return PyLong_FromUnsignedLongLong(drgn_type_size(self->type));
+	return PyLong_FromUint64(drgn_type_size(self->type));
 }
 
 static PyObject *DrgnType_get_length(DrgnType *self)
@@ -129,7 +128,7 @@ static PyObject *DrgnType_get_length(DrgnType *self)
 				    drgn_type_kind_str(self->type));
 	}
 	if (drgn_type_is_complete(self->type))
-		return PyLong_FromUnsignedLongLong(drgn_type_length(self->type));
+		return PyLong_FromUint64(drgn_type_length(self->type));
 	else
 		Py_RETURN_NONE;
 }
@@ -182,8 +181,7 @@ static TypeMember *TypeMember_wrap(PyObject *parent,
 				   struct drgn_type_member *member,
 				   uint64_t bit_offset)
 {
-	TypeMember *py_member =
-		(TypeMember *)TypeMember_type.tp_alloc(&TypeMember_type, 0);
+	TypeMember *py_member = call_tp_alloc(TypeMember);
 	if (!py_member)
 		return NULL;
 
@@ -198,7 +196,7 @@ static TypeMember *TypeMember_wrap(PyObject *parent,
 		Py_INCREF(Py_None);
 		py_member->name = Py_None;
 	}
-	py_member->bit_offset = PyLong_FromUnsignedLongLong(bit_offset);
+	py_member->bit_offset = PyLong_FromUint64(bit_offset);
 	if (!py_member->bit_offset)
 		goto err;
 	return py_member;
@@ -310,10 +308,7 @@ static PyObject *DrgnType_get_parameters(DrgnType *self)
 
 	for (i = 0; i < num_parameters; i++) {
 		struct drgn_type_parameter *parameter = &parameters[i];
-		TypeParameter *item;
-
-		item = (TypeParameter *)TypeParameter_type.tp_alloc(&TypeParameter_type,
-								    0);
+		TypeParameter *item = call_tp_alloc(TypeParameter);
 		if (!item)
 			goto err;
 		PyTuple_SET_ITEM(parameters_obj, i, (PyObject *)item);
@@ -367,9 +362,7 @@ static PyObject *DrgnType_get_template_parameters(DrgnType *self)
 			&template_parameters[i];
 
 		TypeTemplateParameter *item =
-			(TypeTemplateParameter *)
-			TypeTemplateParameter_type.tp_alloc(&TypeTemplateParameter_type,
-							    0);
+			call_tp_alloc(TypeTemplateParameter);
 		if (!item)
 			goto err;
 		PyTuple_SET_ITEM(template_parameters_obj, i, (PyObject *)item);
@@ -751,6 +744,8 @@ static PyMethodDef DrgnType_methods[] = {
 	 drgn_Type_member_DOC},
 	{"has_member", (PyCFunction)DrgnType_has_member,
 	 METH_VARARGS | METH_KEYWORDS, drgn_Type_has_member_DOC},
+	{"_repr_pretty_", (PyCFunction)repr_pretty_from_str,
+	 METH_VARARGS | METH_KEYWORDS},
 	{},
 };
 
@@ -1086,17 +1081,15 @@ static void TypeMember_dealloc(TypeMember *self)
 
 static PyObject *TypeMember_get_offset(TypeMember *self, void *arg)
 {
-	unsigned long long bit_offset;
-
-	bit_offset = PyLong_AsUnsignedLongLong(self->bit_offset);
-	if (bit_offset == (unsigned long long)-1 && PyErr_Occurred())
+	uint64_t bit_offset = PyLong_AsUint64(self->bit_offset);
+	if (bit_offset == (uint64_t)-1 && PyErr_Occurred())
 		return NULL;
 	if (bit_offset % 8) {
 		PyErr_SetString(PyExc_ValueError,
 				"member is not byte-aligned");
 		return NULL;
 	}
-	return PyLong_FromUnsignedLongLong(bit_offset / 8);
+	return PyLong_FromUint64(bit_offset / 8);
 }
 
 static PyObject *TypeMember_get_bit_field_size(TypeMember *self, void *arg)
@@ -1105,7 +1098,7 @@ static PyObject *TypeMember_get_bit_field_size(TypeMember *self, void *arg)
 	if (!object)
 		return NULL;
 	if (object->obj.is_bit_field)
-		return PyLong_FromUnsignedLongLong(object->obj.bit_size);
+		return PyLong_FromUint64(object->obj.bit_size);
 	else
 		Py_RETURN_NONE;
 }
@@ -1638,9 +1631,8 @@ static int unpack_member(struct drgn_compound_type_builder *builder,
 			return -1;
 	}
 
-	unsigned long long bit_offset =
-		PyLong_AsUnsignedLongLong(member->bit_offset);
-	if (bit_offset == (unsigned long long)-1 && PyErr_Occurred())
+	uint64_t bit_offset = PyLong_AsUint64(member->bit_offset);
+	if (bit_offset == (uint64_t)-1 && PyErr_Occurred())
 		return -1;
 
 	union drgn_lazy_object object;
@@ -1883,14 +1875,13 @@ static int unpack_enumerator(struct drgn_enum_type_builder *builder,
 
 	struct drgn_error *err;
 	if (is_signed) {
-		long long svalue = PyLong_AsLongLong(enumerator->value);
+		int64_t svalue = PyLong_AsInt64(enumerator->value);
 		if (svalue == -1 && PyErr_Occurred())
 			return -1;
 		err = drgn_enum_type_builder_add_signed(builder, name, svalue);
 	} else {
-		unsigned long long uvalue =
-			PyLong_AsUnsignedLongLong(enumerator->value);
-		if (uvalue == (unsigned long long)-1 && PyErr_Occurred())
+		uint64_t uvalue = PyLong_AsUint64(enumerator->value);
+		if (uvalue == (uint64_t)-1 && PyErr_Occurred())
 			return -1;
 		err = drgn_enum_type_builder_add_unsigned(builder, name,
 							  uvalue);
