@@ -1,5 +1,5 @@
 // Copyright (c) Meta Platforms, Inc. and affiliates.
-// SPDX-License-Identifier: GPL-3.0-or-later
+// SPDX-License-Identifier: LGPL-2.1-or-later
 
 /**
  * @file
@@ -14,6 +14,7 @@
 
 #include <assert.h>
 #include <stdbool.h>
+#include <stddef.h>
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
@@ -49,14 +50,12 @@
  */
 struct drgn_register_state {
 	/**
-	 * Cached @ref drgn_debug_info_module that contains the program counter.
+	 * Cached @ref drgn_module that contains the program counter.
 	 *
-	 * This is `NULL` if the program counter is not known, if the containing
-	 * module could not be found, or if the containing module's platform
-	 * does not match the program's platform (in which case we can't use it
-	 * anyways).
+	 * This is `NULL` if the program counter is not known or if the
+	 * containing module could not be found.
 	 */
-	struct drgn_debug_info_module *module;
+	struct drgn_module *module;
 	/** Total size of registers allocated in @ref drgn_register_state::buf. */
 	uint32_t regs_size;
 	/** Number of registers allocated in @ref drgn_register_state::buf. */
