@@ -1,16 +1,16 @@
 # Copyright (c) Meta Platforms, Inc. and affiliates.
-# SPDX-License-Identifier: GPL-3.0-or-later
+# SPDX-License-Identifier: LGPL-2.1-or-later
 
-import platform
 import re
 import unittest
 
 from drgn.helpers.linux.boot import pgtable_l5_enabled
 from tests.linux_kernel import LinuxKernelTestCase
+from util import NORMALIZED_MACHINE_NAME
 
 
 class TestBoot(LinuxKernelTestCase):
-    @unittest.skipUnless(platform.machine() == "x86_64", "machine is not x86_64")
+    @unittest.skipUnless(NORMALIZED_MACHINE_NAME == "x86_64", "machine is not x86_64")
     def test_pgtable_l5_enabled(self):
         with open("/proc/cpuinfo", "r") as f:
             self.assertEqual(
