@@ -1,5 +1,5 @@
 # Copyright (c) Meta Platforms, Inc. and affiliates.
-# SPDX-License-Identifier: GPL-3.0-or-later
+# SPDX-License-Identifier: LGPL-2.1-or-later
 
 from drgn import (
     Language,
@@ -14,7 +14,12 @@ from drgn import (
     offsetof,
     sizeof,
 )
-from tests import DEFAULT_LANGUAGE, MockProgramTestCase, mock_program
+from tests import (
+    DEFAULT_LANGUAGE,
+    MockProgramTestCase,
+    assertReprPrettyEqualsStr,
+    mock_program,
+)
 
 
 class TestType(MockProgramTestCase):
@@ -973,6 +978,10 @@ class TestType(MockProgramTestCase):
             "prog.void_type(language=Language.CPP)",
         )
 
+    def test__repr_pretty_(self):
+        t = self.prog.void_type()
+        assertReprPrettyEqualsStr(t)
+
     def test_different_programs_compound(self):
         self.assertRaisesRegex(
             ValueError,
